/*==============================================================================


       extra.cpp - addon file to main


==============================================================================*/
#include "sys.h"         // eBookMan include files
#include "gui.h"

extern "C"
{
#include "ereader_hostio.h"
#include "evnt_fun.h"
#include "lcd.h"
#include "timer.h"
#include "ereader_hostio.h"
#include "piezo.h"
#include "ebm_object.h"
#include "stdlib.h"
}


#include "main_fff.h"
#include "enter_name_fff.h"


enum
{
  MainWindow_Territory=110,
};

	
char *Help_text = "First player (you) are white.  Second player is black.  Third player is gray and is always played by the EBM.
Game starts with each player placeing their initial armies.  Then player one selects a 'from' and 'to' by clicking on the desired territories.
Press the attack button to attack.  After a territory is concurred you may press the move button to move additional armies into that territory.
You may also move armies at the end of your turn.  Hit the next button to end your turn.
In the menu you can select a second 'human' player, change the 'skill' level and adjust 'computer' speed.
Have fun!
********************
WhitWareInc Software [whitwareinc@yahoo.com]
Version 2.01
"__DATE__"-"__TIME__"
";

#define RND_TIMER	100

enum {
	MSG_RANDOM=MSG_USER+1,
	MSG_GAME_LOOP,
};

enum {
	GS_SETUP=0,
	GS_START_ROUND,
	GS_PLACE,
	GS_SELECT_FROM,
	GS_SELECT_TO,
	GS_ATTACK,
	GS_MOVE,
	GS_NEXT,
};


MainWindow *f_MainWindow;
EnterNameWindow *f_EnterNameWindow;

char Title_bar[50]="";

typedef struct territory_struct {
	U16	Id;
	char	Name[20];
	U16	X;
	U16	Y;
	U16	Continent;
	U16	Borders[10];
	U16	Player;
	U16	Armies;
};

enum CONTINENTS {
	NORTH_AMERICA=0,
	SOUTH_AMERICA,
	SOUTH_AFRICA,
	EUROPE,
	ASIA,
	AUSTRALIA,
};

#define MAX_TERRITORIES 42
struct territory_struct Territory[MAX_TERRITORIES] = {
	{1, {"Alaska"}, 5, 30, NORTH_AMERICA, {2, 4, 36, 0}, 0, 0}, 
	{2, {"Northwest"}, 26, 32, NORTH_AMERICA, {1, 3, 4, 5, 0}, 0, 0}, 
	{3, {"Greenland"}, 80, 10, NORTH_AMERICA, {2, 5, 6, 20, 0}, 0, 0}, 
	{4, {"Alberta"}, 28, 50, NORTH_AMERICA, {1, 2, 5, 7, 0}, 0, 0}, 
	{5, {"Ontario"}, 44, 53, NORTH_AMERICA, {2, 3, 4, 6, 7, 8, 0}, 0, 0}, 
	{6, {"Quebec"}, 64, 56, NORTH_AMERICA, {3, 5, 8, 0}, 0, 0}, 
	{7, {"W. U.S."}, 34, 74, NORTH_AMERICA, {4, 5, 6, 8, 9, 0}, 0, 0}, 
	{8, {"E. U.S."}, 54, 78, NORTH_AMERICA, {5, 6, 7, 9, 0}, 0, 0}, 
	{9, {"Central Amarica"}, 38, 100, NORTH_AMERICA, {7, 8, 10, 0}, 0, 0}, 
	{10, {"Venezuela"}, 58, 112, SOUTH_AMERICA, {9, 11, 13, 0}, 0, 0}, 
	{11, {"Peru"}, 50, 136, SOUTH_AMERICA, {10, 12, 13, 0}, 0, 0}, 
	{12, {"Argentina"}, 62, 162, SOUTH_AMERICA, {11, 13, 42, 0}, 0, 0}, 
	{13, {"Brazil"}, 76, 130, SOUTH_AMERICA, {10, 11, 12, 14, 0}, 0, 0}, 
	{14, {"N. Africa"}, 104, 100, SOUTH_AFRICA, {13, 15, 18, 19, 22, 24, 0}, 0, 0}, 
	{15, {"Congo"}, 120, 125, SOUTH_AFRICA, {14, 16, 18, 0}, 0, 0}, 
	{16, {"S. Africa"}, 122, 144, SOUTH_AFRICA, {15, 17, 18, 0}, 0, 0}, 
	{17, {"Madagascar"}, 140, 142, SOUTH_AFRICA, {16, 18, 0}, 0, 0}, 
	{18, {"E. Africa"}, 132, 112, SOUTH_AFRICA, {14, 15, 16, 17, 19, 27, 0}, 0, 0}, 
	{19, {"Egypt"}, 122, 88, SOUTH_AFRICA, {14, 18, 24, 27, 0}, 0, 0}, 
	{20, {"Iceland"}, 98, 28, EUROPE, {3, 21, 25, 0}, 0, 0}, 
	{21, {"Great Britain"}, 96, 56, EUROPE, {20, 22, 23, 25, 0}, 0, 0}, 
	{22, {"W. Europe"}, 98, 70, EUROPE, {14, 21, 23, 24, 0}, 0, 0}, 
	{23, {"N. Europe"}, 112, 58, EUROPE, {21, 22, 24, 25, 26, 0}, 0, 0}, 
	{24, {"S. Europe"}, 118, 70, EUROPE, {14, 19, 22, 23, 26, 27, 0}, 0, 0}, 
	{25, {"Scandinavia"}, 116, 24, EUROPE, {20, 21, 23, 26, 0}, 0, 0}, 
	{26, {"Ukraine"}, 132, 50, EUROPE, {23, 24, 25, 27, 29, 30, 0}, 0, 0}, 
	{27, {"Middle East"}, 136, 84, EUROPE, {18, 19, 24, 26, 30, 31, 0}, 0, 0}, 
	{28, {"Taymyr"}, 166, 16, ASIA, {29, 32, 33, 34, 0}, 0, 0}, 
	{29, {"Yamal-Nemets"}, 152, 42, ASIA, {26, 28, 30, 34, 0}, 0, 0}, 
	{30, {"Kazakhstan"}, 148, 68, ASIA, {26, 27, 29, 31, 35, 0}, 0, 0}, 
	{31, {"India"}, 156, 96, ASIA, {27, 30, 35, 38, 0}, 0, 0}, 
	{32, {"Yakut"}, 188, 26, ASIA, {28, 33, 36, 0}, 0, 0}, 
	{33, {"Burat"}, 182, 54, ASIA, {28, 32, 34, 36, 0}, 0, 0}, 
	{34, {"Mongolia"}, 176, 68, ASIA, {33, 35, 36, 37, 0}, 0, 0}, 
	{35, {"China"}, 172, 84, ASIA, {30, 31, 34, 38, 0}, 0, 0}, 
	{36, {"Koryak"}, 216, 30, ASIA, {1, 32, 33, 34, 37, 0}, 0, 0}, 
	{37, {"Japan"}, 202, 80, ASIA, {34, 36, 0}, 0, 0}, 
	{38, {"Siam"}, 172, 108, ASIA, {31, 35, 39, 0}, 0, 0}, 
	{39, {"Indonesia"}, 172, 124, AUSTRALIA, {38, 40, 41, 0}, 0, 0}, 
	{40, {"New Guinea"}, 208, 126, AUSTRALIA, {39, 41, 42, 0}, 0, 0}, 
	{41, {"W. Austrailia"}, 180, 150, AUSTRALIA, {39, 40, 42, 0}, 0, 0}, 
	{42, {"E. Austrailia"}, 204, 154, AUSTRALIA, {12, 40, 41, 0}, 0, 0}, 
};

CLabel *Territory_label[MAX_TERRITORIES];
char Territory_label_str[MAX_TERRITORIES][20];

#define MAX_PLAYERS 3
#define NUM_INITIAL_ARMIES 35 /* 35 */
enum
{
	HUMAN=0,
	COMPUTER,
};
enum
{
	HARD=0,
	MEDIUM,
	EASY,
};
enum
{
	FAST_SPEED=1,
	MEDIUM_SPEED=5,
	SLOW_SPEED=10,
};

U16 Player_type[MAX_PLAYERS] = {HUMAN, COMPUTER, COMPUTER};
U16 Player_out[MAX_PLAYERS];
U16 Running, Game_state, Current_player, Num_armies_placed, Num_armies_to_place;
U16 From, To, Just_concored, Turns;
U16 Skill, Humans, Last_human_territory, Speed;

char Skill_str[3][10] = {
	"Hard   ",
	"Medium",
	"Easy   ",
};

U16 Computer_turn;

typedef struct score_struct {U32 score; U16 skill; char name[50];};


/* ------------------------------------------------------------------------- */


void HighScore (U32 score)
{
	struct score_struct *high_scores, tmp_score;
	size_t max_size=ebo_roundup(sizeof(score_struct)*10);

	static const ebo_name_t file_name = { "","WhitWare","Risk","ini" };
	char *file_ptr=(char *)ebo_roundup(OS_availaddr);
	OS_availaddr=(U32)file_ptr + max_size;

	U16 num_scores, i, j;
	char high_scores_str[500], *name_p;
	
	if (ebo_mapin(&file_name,0,file_ptr,&max_size,1) < 0)
	{
		if ( ebo_new(&file_name,max_size,1) < 0)
      			return;
		else if (ebo_mapin(&file_name,0,file_ptr,&max_size,1) < 0)
			return;
	}
	high_scores = (score_struct *)file_ptr;
	num_scores = 0;
	high_scores_str[0] = 0;
	while ( high_scores[num_scores].score )
	{
		num_scores++;
	}

	if(score)
	{
		if(num_scores > 9)
			num_scores = 9;
		high_scores[num_scores].score = score;
		high_scores[num_scores].skill = Skill;
		name_p = f_EnterNameWindow->EnterName->GetText();
		strcpy(high_scores[num_scores].name, name_p);
		free(name_p);
		f_MainWindow->SetFocus(f_MainWindow);
		num_scores++;
	}
	// qsort(high_scores, (size_t)num_scores, (size_t)sizeof(score_struct), (int (*)(...))ScoreCmp);
	for (i = 0; i < num_scores; i++)
	{
		for (j = i+1; j < num_scores; j++)
			if((high_scores[j].skill < high_scores[i].skill) ||
			   ((high_scores[j].skill == high_scores[i].skill) &&
			    (high_scores[j].score < high_scores[i].score)))
			{
				tmp_score = high_scores[i];
				high_scores[i] = high_scores[j];
				high_scores[j] = tmp_score;
			}
	}		
	sprintf(&high_scores_str[strlen(high_scores_str)],"*************\n  High Scores\n*************\n");
	for (i = 0; i < num_scores; i++)
	{
		sprintf(&high_scores_str[strlen(high_scores_str)],"%6ld | %s | %s\n",
			high_scores[i].score, Skill_str[high_scores[i].skill], high_scores[i].name);
	}

	GUI_Alert(ALERT_INFO, high_scores_str);

	ebo_unmap(file_ptr, max_size);
}

void UpdateTerritories(void)
{
	U16 i;
	
	for (i = 0; i < MAX_TERRITORIES; i++)
	{
		sprintf(Territory_label_str[i], "%d", Territory[i].Armies);
		Territory_label[i]->SetLabel(Territory_label_str[i]);
		if(Territory[i].Player == 1)
		{
			Territory_label[i]->SetForegroundColor(COLOR_BLACK);
			Territory_label[i]->SetBackgroundColor(COLOR_WHITE);
		}
		else if(Territory[i].Player == 2)
		{
			Territory_label[i]->SetForegroundColor(COLOR_WHITE);
			Territory_label[i]->SetBackgroundColor(COLOR_BLACK);
		}
		else
		{
			Territory_label[i]->SetForegroundColor(COLOR_WHITE);
			Territory_label[i]->SetBackgroundColor(COLOR_GRAY53);
		}
	}
}

void NewGame(void)
{
	U16 i, j;

	for (i = 0; i < MAX_TERRITORIES; i++)
	{
		Territory[i].Player = 0;
	}
	for (j = 0; j < MAX_TERRITORIES; j++)
	{
		i = ((rand()%(MAX_TERRITORIES*1000)/1000));
		while(Territory[i].Player)
			if(++i >= MAX_TERRITORIES)
				i = 0;
		Territory[i].Player = (j%3)+1;
		Territory[i].Armies = 1;
	}
	for (i = 0; i < MAX_PLAYERS; i++)
		Player_out[i] = 0;
	From = 0;
	To = 0;
	Turns = 0;
	Last_human_territory = 0;
	UpdateTerritories();
	Current_player = 0;
	Num_armies_placed = 0;
	Just_concored = 0;
	Num_armies_to_place = NUM_INITIAL_ARMIES;
	sprintf(Title_bar,"Player-%d place %d/%d",Current_player+1, Num_armies_placed+1, Num_armies_to_place);
	f_MainWindow->SetTitle(Title_bar);
	Game_state = GS_SETUP;
	Running = 1;
        GUI_EventMessage( (MSG_TYPE)(MSG_GAME_LOOP), f_MainWindow, 0 );
}
//------------------------------------------------------------------------------
void EnterNameWindow::Close( void )
{
	HighScore(Turns);

	return CWindow::Close();
}

void PlaceArmy (U16 i)
{
	Territory[i].Armies++;
	UpdateTerritories();
	sprintf(Title_bar,"Player-%d place %d/%d",Current_player+1, Num_armies_placed+1, Num_armies_to_place);
	f_MainWindow->SetTitle(Title_bar);
}

void Select (U16 i)
{
	if(Game_state == GS_SELECT_FROM)
	{
		From = i;
		To = 0xff;
	}
	else 
		To = i;
	
	if(To == 0xff)
		sprintf(Title_bar,"%d: %s",Current_player+1, Territory[From].Name);
	else
		sprintf(Title_bar,"%d: %s > %s",Current_player+1, Territory[From].Name, Territory[To].Name);
	f_MainWindow->SetTitle(Title_bar);
}

U16 CalcNumContinent (U16 c, U16 p)
{
	U16 i, j;
	
	j = 0;
	for (i = 0; i < MAX_TERRITORIES; i++)
	{
		if((Territory[i].Player == p+1) &&
		   (Territory[i].Continent == c))
		{
			j++;
		}
	}
	return(j);
}

U16 CalcNumTerritories (U16 p)
{
	U16 i, j;
	
	j = 0;
	for (i = 0; i < MAX_TERRITORIES; i++)
	{
		if(Territory[i].Player == p+1)
		{
			j++;
		}
	}
	return(j);
}

U16 CalcNumArmies (U16 p)
{
	U16 i;
	
	i = (CalcNumTerritories (p)) / 3;
	if (i < 3)
		i = 3;
	if(CalcNumContinent(SOUTH_AMERICA, p) == 4)
		i +=2;
	if(CalcNumContinent(AUSTRALIA, p) == 4)
		i +=2;
	if(CalcNumContinent(SOUTH_AFRICA, p) == 6)
		i +=3;
	if(CalcNumContinent(NORTH_AMERICA, p) == 9)
		i +=5;
	if(CalcNumContinent(EUROPE, p) == 8)
		i +=5;
	if(CalcNumContinent(ASIA, p) == 11)
		i +=7;
	return (i);
}

U16 Max_dice (U16 *dice)
{
	U16 i, j;
	
	i = j = 0;
	while(dice[i]!= 0xff)
	{
		if (dice[i] > dice[j])
			j = i;
		i++;
	}
	i = dice[j];
	dice[j]= 0;
	return(i);
}

void Attack ( void )
{
	U16 num_from_dice, num_to_dice;
	U16 from_dice[5], to_dice[5];
	U16 i, j, to_loose, from_loose, min_dice;
	
	if((Territory[From].Player != (Current_player+1)) ||
	   (Territory[From].Armies < 2))
	{
		GUI_Alert(ALERT_INFO, "Can't attack from that territory");
		return;
	}
	if(Territory[To].Player == (Current_player+1))
	{
		GUI_Alert(ALERT_INFO, "Can't attack that territory");
		return;
	}

	num_from_dice = 3;
	if (Territory[From].Armies < 4)
		num_from_dice = Territory[From].Armies-1;
	num_to_dice = 2;
	if (Territory[To].Armies < 2)
		num_to_dice = 1;
	for (i = 0; i < num_from_dice; i++)
		from_dice[i] = ((rand()%6000)/1000)+1;
	from_dice[i] = 0xff;
	for (i = 0; i < num_to_dice; i++)
		to_dice[i] = ((rand()%6000)/1000)+1;
	to_dice[i] = 0xff;
	to_loose=0;
	from_loose=0;
	min_dice = num_from_dice;
	if(num_to_dice < min_dice)
		min_dice = num_to_dice;
	for (i = 0; i < min_dice; i++)
	{
		if(Max_dice(from_dice) > Max_dice(to_dice))
			to_loose++;
		else
			from_loose++;
	}
	Territory[From].Armies -=from_loose;
	Territory[To].Armies -=to_loose;
	if(!Territory[To].Armies)
	{
		Territory[From].Armies -=1;
		Territory[To].Armies =1;
		i = Territory[To].Player;
		Territory[To].Player = Current_player+1;
		j = CalcNumTerritories(i-1);
		if (!j)
			Player_out[i-1] = 1;
		j = 0;
		for(i = 0; i < MAX_PLAYERS; i++)
			if(Player_out[i])
				j++;
		if (j >=2)
		{
			Running = 0;
			UpdateTerritories();
			GUI_UpdateNow();
			if(Player_type[Current_player] == HUMAN)
			{
				f_EnterNameWindow = new EnterNameWindow();
			}
			else
				GUI_Alert(ALERT_INFO, "EBM is the winner!");
			return;
		}	
		else
			Just_concored = 1;
	}
	UpdateTerritories();
}

U16 Move ( U16 count)
{
	while(count--)
	{
		if((Territory[From].Player != (Current_player+1)) ||
		   (Territory[From].Armies < 2))
		{
			GUI_Alert(ALERT_INFO, "Can't move from that territory");
			return 0;
		}
		if(Territory[To].Player != (Current_player+1))
		{
			GUI_Alert(ALERT_INFO, "Can't move to that territory");
			return 0;
		}
	
		if(Territory[From].Armies > 1)
		{
			Territory[From].Armies -=1;
			Territory[To].Armies +=1;
		}
	}
	UpdateTerritories();
	return 1;
}

U16 HasHumanEnemy(U16 t)
{
	U16 i;
	i = 0;
	while((Territory[t].Borders[i]) &&
	      (Player_type[(Territory[Territory[t].Borders[i]-1].Player)-1] == COMPUTER))
	      i++;
	return (Territory[t].Borders[i]);
}

U16 HasEnemy(U16 t)
{
	U16 i;
	i = 0;
	while((Territory[t].Borders[i]) &&
	      (Territory[Territory[t].Borders[i]-1].Player == Territory[t].Player))
	      i++;
	return (Territory[t].Borders[i]);
}

//------------------------------------------------------------------------------
S32 MainWindow::MsgHandler( MSG_TYPE type, CViewable *from, S32 data )
{
	U16 i, j;
	S16 x, y;
	RECT target_rect;
	static U16 difficulty = 0;

// message handler

	switch ( type )
	{
		case MSG_KEY:
			switch ( data )
			{
				case K_MENU: // process request to show menu
					if(Running)
						Running = 2;
					m_menu1->Show();
					return 1;

				case K_UP:
				case K_PAGE_UP:
				case K_DOWN:
				case K_PAGE_DOWN:
				case K_LEFT:
				case K_RIGHT:
					return 1;

				case K_JOG_UP:
				case K_JOG_ENTER:
				case K_JOG_DOWN:
					return 1;

				default:
					return 0;
			}

		case MSG_PEN_DOWN:
            		if ((Running==1) && (Player_type[Current_player] == HUMAN))
	       		{
				y = data & 0xffff;
				x = (data >> 16) & 0xffff;
				for (i = 0; i < MAX_TERRITORIES; i++)
				{
					Territory_label[i]->GetRect(&target_rect);
					if(GUI_PointInRect(&target_rect, x, y))
					{
						if (Game_state == GS_SETUP)
						{
							if(Territory[i].Player == (Current_player+1))
							{
					        		if(++Current_player >= MAX_PLAYERS)
					        		{
					        			Current_player = 0;
									if(++Num_armies_placed >= Num_armies_to_place)
										Game_state = GS_START_ROUND;
					        		}
								PlaceArmy(i);
								Last_human_territory = i;
								GUI_EventMessage( (MSG_TYPE)(MSG_GAME_LOOP), f_MainWindow, 0 );
							}
						}
						else if (Game_state == GS_PLACE)
						{
							if(Territory[i].Player == (Current_player+1))
							{
								PlaceArmy(i);
								Last_human_territory = i;
								if(++Num_armies_placed >= Num_armies_to_place)
								{
									Game_state = GS_SELECT_FROM;
									sprintf(Title_bar,"Player-%d select",Current_player+1);
									f_MainWindow->SetTitle(Title_bar);
								}
							}
						}
						else if ((Game_state == GS_SELECT_FROM) || (Game_state == GS_ATTACK))
						{
							Just_concored = 0;
							Game_state = GS_SELECT_FROM;
							if(Territory[i].Player == (Current_player+1))
							{
								Select(i);
								Game_state = GS_SELECT_TO;
							}
						}
						else if (Game_state == GS_SELECT_TO)
						{
							j = 0;
					
							while((Territory[From].Borders[j]) &&
							      ((Territory[From].Borders[j]-1) != i))
					      			j++;
						
							if(Territory[From].Borders[j])
							{
								Select(i);
								Game_state = GS_ATTACK;
							}
							else
							{
								Game_state = GS_SELECT_FROM;
								sprintf(Title_bar,"Player-%d select",Current_player+1);
								f_MainWindow->SetTitle(Title_bar);
							}
						}
						i = MAX_TERRITORIES;
					}
				}
			}
			else if(Running == 2)
			{
				Running = 1;
				GUI_EventMessage( (MSG_TYPE)(MSG_GAME_LOOP), f_MainWindow, 0 );
				SetTimer(RND_TIMER, FALSE, 0);
			}
			return 1;

		case MSG_BUTTON_SELECT:
			if((Running==1) && (Player_type[Current_player] == HUMAN))
			{
				switch( from->GetID() )
				{
					case MainWindow_CardButton:
						break;
					case MainWindow_MoveButton:
						if ((Game_state == GS_MOVE) ||(Game_state == GS_ATTACK))
						{
							if((Move(1)) && (!Just_concored))
								Game_state = GS_MOVE;
						}
						break;
					case MainWindow_AttackButton:
		            			if (Game_state == GS_ATTACK)
		            			{
							Attack();
						}
						break;
					case MainWindow_DoneButton:
						Game_state = GS_NEXT;
					        GUI_EventMessage( (MSG_TYPE)(MSG_GAME_LOOP), f_MainWindow, 0 );
						break;
					
					default:
						return 0;
				}
			}
			else if(Running == 2)
			{
				Running = 1;
				GUI_EventMessage( (MSG_TYPE)(MSG_GAME_LOOP), f_MainWindow, 0 );
				SetTimer(RND_TIMER, FALSE, 0);
			}
			return 1;

            	case MSG_TIMER:
            		GUI_EventMessage( (MSG_TYPE)(MSG_RANDOM), f_MainWindow, 0 );
            		return 1;
            		
            	case MSG_RANDOM:
  	  	 	for ( i = 0; i < 100; i++)
			 	rand();
			if(Computer_turn)
			{
				if(!(--Computer_turn))
					GUI_EventMessage( (MSG_TYPE)(MSG_GAME_LOOP), f_MainWindow, 0 );
			}
			SetTimer(RND_TIMER, FALSE, 0);
 			return 1;

		case MSG_GAME_LOOP:
            	   if (Running==1)
            	   {
			if (Game_state == GS_NEXT)
			{
				Turns++;
				++difficulty;
				do
				{
					if(++Current_player >= MAX_PLAYERS)
						Current_player = 0;
				} while (!CalcNumTerritories(Current_player));
				Game_state = GS_START_ROUND;
			}
			if(Game_state == GS_START_ROUND)
			{
				Just_concored = 0;
				Num_armies_placed = 0;
				Num_armies_to_place = CalcNumArmies(Current_player);
				Game_state = GS_PLACE;
				sprintf(Title_bar,"Player-%d place %d/%d",Current_player+1, Num_armies_placed+1, Num_armies_to_place);
				f_MainWindow->SetTitle(Title_bar);
			}
            		if (Player_type[Current_player] == COMPUTER)
	       		{
				if (Game_state == GS_SETUP)
				{
					if(++difficulty > ((Skill*2)+1))
						difficulty = 0;
					if((!difficulty) && (Humans == 1))
						i = Last_human_territory;
					else
						i = ((rand()%(MAX_TERRITORIES*1000)/1000));
					
					j = 0;
					while((j < MAX_TERRITORIES) && 
						((Territory[i].Player != (Current_player+1)) || !HasHumanEnemy(i)))
					{
						if(++i >= MAX_TERRITORIES)
							i = 0;
						j++;
					}
					if(j >= MAX_TERRITORIES)
					{
						j = 0;
						while((j < MAX_TERRITORIES) && 
							((Territory[i].Player != (Current_player+1)) || !HasEnemy(i)))
						{
							if(++i >= MAX_TERRITORIES)
								i = 0;
							j++;
						}
					}
					
					if(j >= MAX_TERRITORIES)
					{
						while((Territory[i].Player) != (Current_player+1))
							if(++i >= MAX_TERRITORIES)
								i = 0;
					}
		        		if(++Current_player >= MAX_PLAYERS)
		        		{
		        			Current_player = 0;
						if(++Num_armies_placed >= Num_armies_to_place)
							Game_state = GS_START_ROUND;
		        		}
					PlaceArmy(i);
				}
				else if (Game_state == GS_PLACE)
				{
					if(++difficulty > Skill)
						difficulty = 0;
					if((!difficulty) && (Humans == 1))
						i = Last_human_territory;
					else
						i = ((rand()%(MAX_TERRITORIES*1000)/1000));
					
					j = 0;
					while((j < MAX_TERRITORIES) && 
						((Territory[i].Player != (Current_player+1)) || !HasHumanEnemy(i)))
					{
						if(++i >= MAX_TERRITORIES)
							i = 0;
						j++;
					}
					if(j >= MAX_TERRITORIES)
					{
						j = 0;
						while((j < MAX_TERRITORIES) && 
							((Territory[i].Player != (Current_player+1)) || !HasEnemy(i)))
						{
							if(++i >= MAX_TERRITORIES)
								i = 0;
							j++;
						}
					}
					if(j < MAX_TERRITORIES)
					{
						PlaceArmy(i);
						if(++Num_armies_placed >= Num_armies_to_place)
						{
							Game_state = GS_SELECT_FROM;
							sprintf(Title_bar,"Player-%d select",Current_player+1);
							f_MainWindow->SetTitle(Title_bar);
						}
					}
				}
				else if (Game_state == GS_SELECT_FROM)
				{
					i = ((rand()%(MAX_TERRITORIES*1000)/1000));
					j = 0;
					while(((Territory[i].Player != (Current_player+1)) || (Territory[i].Armies <= 3) || !HasHumanEnemy(i)) &&
					      (j < MAX_TERRITORIES))
					{
						if(++i >= MAX_TERRITORIES)
							i = 0;
						j++;
					}
					if(j >= MAX_TERRITORIES)
					{
						j = 0;
						while(((Territory[i].Player != (Current_player+1)) || (Territory[i].Armies <= 3) || !HasEnemy(i)) &&
						      (j < MAX_TERRITORIES))
						{
							if(++i >= MAX_TERRITORIES)
								i = 0;
							j++;
						}
					}
					if(j >= MAX_TERRITORIES)
					{
						Game_state = GS_NEXT;
					}
					else
					{
						Select(i);
						Game_state = GS_SELECT_TO;
					}
				}
				else if (Game_state == GS_SELECT_TO)
				{
					i = 0;
					
					if(++difficulty > Skill)
						difficulty = 0;
					while((Territory[From].Borders[i]) &&
					      (((Territory[Territory[From].Borders[i]-1].Player) == (Current_player+1)) ||
					       ((Player_type[(Territory[Territory[From].Borders[i]-1].Player)-1] == COMPUTER) && 
					       (!difficulty))))
					      i++;
					
					if(!Territory[From].Borders[i])
					{
						i = 0;
						
						while((Territory[From].Borders[i]) &&
						      ((Territory[Territory[From].Borders[i]-1].Player) == (Current_player+1)))
						      i++;
					}
						
					if(!Territory[From].Borders[i])
					{
						Game_state = GS_NEXT;
					}
					else
					{
						Select(Territory[From].Borders[i]-1);
						Game_state = GS_ATTACK;
					}
				}
				else if (Game_state == GS_ATTACK)
				{
					Attack();
					if((Just_concored) && (Territory[From].Armies > 2))
						if (HasHumanEnemy(From))
							Move(Territory[From].Armies / 2);
						else
							Move(Territory[From].Armies - 2);
					Just_concored = 0;
					Game_state = GS_SELECT_FROM;
				}
				else if (Game_state == GS_MOVE)
				{
					Move(1);
					Game_state = GS_NEXT;
				}
				Computer_turn = Speed;
				SetTimer(RND_TIMER, FALSE, 0);
			}
		   }
		   return 1;
			
		case MSG_MENU_SELECT: // handle menu selection
		{
			switch ((U16)data)
			{
				case IDM_OPTION1: // New Game
					Running = 0;
					NewGame();
					break;
				case IDM_OPTION2: // Num Humans
					Running = 0;
					if (Humans == 1)
					{
						Humans = 2;
						m_menu1->SetRow( 1,IDM_OPTION2,"Num Humans two->one",'n' );
						Player_type[1] = HUMAN;
					}
					else
					{
						Humans = 1;
						m_menu1->SetRow( 1,IDM_OPTION2,"Num Humans one->two",'n' );
						Player_type[1] = COMPUTER;
					}
					break;
				case IDM_OPTION3: // Skill
					Running = 0;
					if (Skill == HARD)
					{
						Skill = MEDIUM;
						m_menu1->SetRow( 2,IDM_OPTION3,"Skill MEDIUM->EASY",'s' );
					}
					else if (Skill == MEDIUM)
					{
						Skill = EASY;
						m_menu1->SetRow( 2,IDM_OPTION3,"Skill EASY->HARD",'s' );
					}
					else
					{
						Skill = HARD;
						m_menu1->SetRow( 2,IDM_OPTION3,"Skill HARD->MEDIUM",'s' );
					}
					break;
				case IDM_OPTION4: // Speed
					if (Speed == FAST_SPEED)
					{
						Speed = MEDIUM_SPEED;
						m_menu1->SetRow( 3,IDM_OPTION4,"Speed MEDIUM->SLOW",'s' );
					}
					else if (Speed == MEDIUM_SPEED)
					{
						Speed = SLOW_SPEED;
						m_menu1->SetRow( 3,IDM_OPTION4,"Speed SLOW->FAST",'s' );
					}
					else
					{
						Speed = FAST_SPEED;
						m_menu1->SetRow( 3,IDM_OPTION4,"Speed FAST->MEDIUM",'s' );
					}
					break;
				case IDM_OPTION5: // High scores
					HighScore(0);
					break;
				case IDM_OPTION6: // Help
					GUI_TextWindow(1,"Help",Help_text);
					break;
				case IDM_OPTION7: // Handwriting help
					GUI_HandwritingHelp(0);
					break;
				case IDM_OPTION8: // Exit
					GUI_Exit();
					break;

			}
			return 1;
		}

		case MSG_OS_NOTIFY:
			switch ((U16) data)
			{
				case NTFY_POWER_ON:
					return 0;
				case NTFY_FOREGROUND:
					if (Running ==2)
					{
						Running = 1;
						GUI_EventMessage( (MSG_TYPE)(MSG_GAME_LOOP), f_MainWindow, 0 );
						SetTimer(RND_TIMER, FALSE, 0);
					}
					return 1;
				case NTFY_BACKGROUND:
					if(Running)
						Running = 2;
					return 1;
				default:
					return 0;
			}

		default:
			return 0;
	}
				
  return CWindow::MsgHandler(type, from, data);
}

//------------------------------------------------------------------------------
void MainWindow::Initialize(void)
{
	U16 i;
	
	CardButton->Hide();
	
// set up menu #1 - with shortcuts
	m_menu1=new CMenu(ID_MENU1,0,32767,0,this);
	m_menu1->SetNumRows(9);
	int row=-1;
	m_menu1->SetRow( ++row,IDM_OPTION1,"New Game",'n' );
	Humans = 1;
	m_menu1->SetRow( ++row,IDM_OPTION2,"Num Humans one->two",'n' );
	Skill = HARD;
	m_menu1->SetRow( ++row,IDM_OPTION3,"Skill HARD->MEDIUM",'s' );
	Speed = FAST_SPEED;
	m_menu1->SetRow( ++row,IDM_OPTION4,"Speed FAST->MEDIUM",'s' );
	m_menu1->SetRow( ++row,IDM_OPTION5,"High Scores",'s' );
	m_menu1->SetRow( ++row,IDM_OPTION6,"Help",'h' );
	m_menu1->SetRow( ++row,IDM_OPTION7,"Handwriting Help",'w' );
	m_menu1->SetSeparatorRow(++row);
	m_menu1->SetRow( ++row,IDM_OPTION8,"Exit",'e' );

	Running = 0;
	for (i = 0; i < MAX_TERRITORIES; i++)
	{
		Territory_label[i]=new CLabel(MainWindow_Territory+i,"",0);
		AddChild(Territory_label[i],Territory[i].X,Territory[i].Y);
		/*
		strcpy(Territory_label_str[i], Territory[i].Name);
		Territory_label[i]->SetLabel(Territory_label_str[i]);
		*/
	}
	
	EnableOSNotify();
	rand();
	SetTimer(RND_TIMER, FALSE, 0);
}

//------------------------------------------------------------------------------
MainWindow::~MainWindow(void)
{
	delete m_menu1; // cleanup
}

//------------------------------------------------------------------------------
S32 GUI_main( MSG_TYPE type, CViewable *object, S32 data )
{
	switch (type)
	{
	case MSG_APP_START:
		if (OS_is_present && hostIO_am_I_the_current_task())
			HOSTIO_INLINE_BREAKPOINT();
		f_MainWindow = new MainWindow();
		f_MainWindow->Initialize(); // do additional setup here
		return 1;
	case MSG_APP_STOP: // prog exit & cleanup
		delete f_MainWindow;
		return 1;
	default:
		break;
	}
	return 0;
};
//------------------------------------------------------------------------------


